package com.fanruan.api.data.open;

import com.fanruan.api.err.TableDataException;
import com.fr.data.AbstractDataModel;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-09-04
 * 二维表模型
 */
public abstract class BaseDataModel extends AbstractDataModel {

    /**
     * 获取数据集的列数
     *
     * @return 列数
     * @throws TableDataException 如果获取数据集列数失败，则抛出此异常
     */
    @Override
    public abstract int getColumnCount() throws TableDataException;

    /**
     * 获取数据集指定列的列名
     *
     * @param rowIndex 列序号
     * @return 类名
     * @throws TableDataException 如果获取列名失败，则抛出此异常
     */
    @Override
    public abstract String getColumnName(int rowIndex) throws TableDataException;

    /**
     * 湖区数据集的行数
     *
     * @return 行数
     * @throws TableDataException 如果获取数据集行数失败，则抛出此异常
     */
    @Override
    public abstract int getRowCount() throws TableDataException;

    /**
     * 获取数据集中指定位置的值
     *
     * @param rowIndex    行
     * @param columnIndex 列
     * @return 值
     * @throws TableDataException 如果获取值失败，则抛出此异常
     */
    @Override
    public abstract Object getValueAt(int rowIndex, int columnIndex) throws TableDataException;

    /**
     * 释放一些构建数据集过程中占用的资源
     *
     * @throws Exception 释放资源时发生错误，则抛出此异常
     */
    @Override
    public abstract void release() throws Exception;
}
