package com.fanruan.api.design.work.component.container;

import com.fanruan.api.design.DesignKit;
import com.fanruan.api.design.ui.component.UISpinner;
import com.fanruan.api.util.StringKit;
import com.fr.design.beans.BasicBeanPane;
import com.fr.design.event.GlobalNameListener;
import com.fr.design.event.GlobalNameObserver;
import com.fr.design.gui.style.NumberDragBar;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019/10/28
 * 可以通过拖动滑动条的方式设置数字的控件
 */
public class NumberDragPane extends BasicBeanPane<Double> implements GlobalNameObserver {
    private static final long serialVersionUID = -8681716725163358247L;
    private NumberDragBar dragBar;
    private UISpinner spinner;
    private boolean isEditing = false;
    private String numberDragPaneName = StringKit.EMPTY;
    private GlobalNameListener globalNameListener = null;

    public NumberDragPane(double minValue, double maxValue) {
        this(minValue, maxValue, 1);
    }

    public NumberDragPane(double minValue, double maxValue, double delta) {
        dragBar = new NumberDragBar((int) minValue, (int) maxValue);
        spinner = new UISpinner(minValue, maxValue, delta, minValue);
        spinner.setGlobalName(DesignKit.i18nText("Fine-Design_Basic_StyleAlignment_Text_Rotation"));
        this.setLayout(new BorderLayout(4, 0));
        this.add(spinner, BorderLayout.EAST);
        this.add(dragBar, BorderLayout.CENTER);
        dragBar.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                if (globalNameListener != null && shouldResponseNameListener()) {
                    globalNameListener.setGlobalName(numberDragPaneName);
                }
                spinner.setValue(dragBar.getValue());
                if (isEditing) {
                    userEvent(updateBean());
                }
            }
        });
        spinner.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                if (globalNameListener != null && shouldResponseNameListener()) {
                    globalNameListener.setGlobalName(numberDragPaneName);
                }
                dragBar.setValue((int) spinner.getValue());
                if (isEditing) {
                    userEvent(updateBean());
                }
            }
        });
    }

    public void userEvent(double value) {

    }

    /**
     * 添加事件
     *
     * @param l 事件
     */
    public void addChangeListener(ChangeListener l) {
        spinner.addChangeListener(l);

    }

    /**
     * 移除事件
     *
     * @param l 事件
     */
    public void removeChangeListener(ChangeListener l) {
        spinner.removeChangeListener(l);
    }

    public void setGlobalName(String name) {
        numberDragPaneName = name;
    }

    @Override
    public void populateBean(Double ob) {
        isEditing = false;
        dragBar.setValue(ob.intValue());
        spinner.setValue(ob);
        isEditing = true;
    }

    @Override
    public Dimension getPreferredSize() {
        Dimension dim = new Dimension();
        dim.width = super.getPreferredSize().width;
        dim.height = super.getPreferredSize().height + 2;
        return dim;
    }

    public void setEnabled(boolean enabled) {
        dragBar.setEnabled(enabled);
        spinner.setEnabled(enabled);
    }

    @Override
    public Double updateBean() {
        return spinner.getValue();
    }

    @Override
    protected String title4PopupWindow() {
        return null;
    }

    /**
     * 注册
     *
     * @param listener 观察者监听事件
     */
    public void registerNameListener(GlobalNameListener listener) {
        globalNameListener = listener;
    }

    /**
     * 是否应该响应
     *
     * @return 是否应该响应
     */
    public boolean shouldResponseNameListener() {
        return true;
    }
}
