package com.fanruan.api.design.work.form.creator;

import com.fanruan.api.log.LogKit;
import com.fr.design.designer.creator.CRPropertyDescriptor;
import com.fr.stable.core.PropertyChangeListener;

import java.beans.IntrospectionException;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-09-10
 * 属性设置帮助类
 */
public class Attribute {

    private CRPropertyDescriptor descriptor;

    /**
     * 创建一个属性描述器
     *
     * @param name      属性名
     * @param beanClass 属性所属的对象类
     * @return 属性描述器
     */
    public static Attribute newAttribute(String name, Class<?> beanClass) {
        return new Attribute(name, beanClass);
    }

    /**
     * 创建一个属性描述器
     *
     * @param name        属性名
     * @param beanClass   属性所属的对象类
     * @param readMethod  属性的读方法
     * @param writeMethod 属性的写方法
     * @return 属性描述器
     */
    public static Attribute newAttribute(String name, Class<?> beanClass, String readMethod, String writeMethod) {
        return new Attribute(name, beanClass, readMethod, writeMethod);
    }

    private Attribute(String name, Class<?> beanClass) {
        try {
            descriptor = new CRPropertyDescriptor(name, beanClass);
        } catch (IntrospectionException e) {
            LogKit.error(e.getMessage(), e);
        }
    }

    private Attribute(String name, Class<?> beanClass, String readMethod, String writeMethod) {
        try {
            descriptor = new CRPropertyDescriptor(name, beanClass, readMethod, writeMethod);
        } catch (IntrospectionException e) {
            LogKit.error(e.getMessage(), e);
        }
    }

    /**
     * 设置属性包含的键值对
     *
     * @param name  属性名
     * @param value 属性值
     */
    public Attribute keyValue(String name, Object value) {
        if (descriptor != null) {
            descriptor.putKeyValue(name, value);
        }
        return this;
    }

    /**
     * 设置属性的显示名字
     *
     * @param i18nName 显示名字
     */
    public Attribute i18n(String i18nName) {
        if (descriptor != null) {
            descriptor.setI18NName(i18nName);
        }
        return this;
    }

    /**
     * 设置属性的编辑类
     *
     * @param clazz 编辑类
     */
    public Attribute editorClass(Class<?> clazz) {
        if (descriptor != null) {
            descriptor.setEditorClass(clazz);
        }
        return this;
    }

    /**
     * 设置属性的渲染类
     *
     * @param clazz 渲染类
     */
    public Attribute rendererClass(Class<?> clazz) {
        if (descriptor != null) {
            descriptor.setRendererClass(clazz);
        }
        return this;
    }

    /**
     * 给属性变化添加监听器
     *
     * @param listener 监听器
     */
    public Attribute listener(PropertyChangeListener listener) {
        if (descriptor != null) {
            descriptor.setPropertyChangeListener(listener);
        }
        return this;
    }

    /**
     * 这个属于内部调用的方法，外部不应该调用
     */
    protected CRPropertyDescriptor toPropertyDescriptor() {
        return descriptor;
    }
}
