package com.fanruan.api.design.work.form.creator;

import com.fanruan.api.util.IOKit;

import javax.swing.*;
import java.awt.*;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-09-10
 */
public class OpacityButton extends JButton {

    private static final float FULL_OPACITY = 1.0F;
    private static final float HALF_OPACITY = 0.4F;

    private String name;
    private String imagePath;
    private float opacity;

    public OpacityButton(String name, String imagePath, float opacity) {
        this.name = name;
        this.imagePath = imagePath;
        this.opacity = opacity;
        this.draw();
    }

    private void draw() {
        ImageIcon imageIcon = (ImageIcon) IOKit.readIcon(imagePath);
        Image img = imageIcon.getImage();
        MediaTracker mt = new MediaTracker(this);
        int w = 21;
        int h = 21;
        mt.addImage(img, 0);
        try {
            mt.waitForAll();
        } catch (InterruptedException ignore) {
            return;
        }

        GraphicsConfiguration gc = new JFrame().getGraphicsConfiguration(); // 本地图形设备
        Image image = gc.createCompatibleImage(w, h, Transparency.TRANSLUCENT);//建立透明画布
        Graphics2D g = (Graphics2D) image.getGraphics(); //在画布上创建画笔

        Composite alpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f); //指定透明度为半透明90%
        g.setComposite(alpha);
        g.drawImage(img, 0, 0, this); //注意是,将image画到g画笔所在的画布上
        g.setColor(Color.black);//设置颜色为黑色
        g.drawString(name, 25, 20);//写字
        g.dispose(); //释放内存

        Composite alpha2 = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, opacity);
        Image image1 = gc.createCompatibleImage(w, h, Transparency.TRANSLUCENT);
        g = (Graphics2D) image1.getGraphics();
        g.setComposite(alpha2);
        g.drawImage(img, 2, 2, this); //改变图像起始位置,产生动态效果
        g.setColor(Color.black);
        g.drawString(name, 25, 20);
        g.dispose();

        this.setIgnoreRepaint(true);
        this.setFocusable(false);//设置没有焦点
        this.setBorder(null);//设置不画按钮边框
        this.setContentAreaFilled(false);//设置不画按钮背景
        this.setIcon(new ImageIcon(image1)); //把刚才生成的半透明image变成ImageIcon,贴到按钮上去
        this.setRolloverIcon(new ImageIcon(image1));
        this.setPressedIcon(new ImageIcon(image));//按下去的图标
    }

    /**
     * 更改组件的可见性
     *
     * @param visible 设置为true表示可见，设置为false表示不可见
     */
    public void makeVisible(boolean visible) {
        this.opacity = visible ? FULL_OPACITY : HALF_OPACITY;
        this.draw();
    }
}
