package com.fanruan.api.report.chart;

import com.fanruan.api.report.chart.field.BaseColumnFieldCollection;
import com.fr.base.chart.cross.FormulaProcessor;
import com.fr.chart.ChartWebParaProvider;
import com.fr.chart.impl.AbstractChartWithData;
import com.fr.extended.chart.HyperLinkPara;
import com.fr.js.NameJavaScriptGroup;
import com.fr.json.JSONObject;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;

import java.awt.Image;

/**
 * @author Bjorn
 * @version 10.0
 * Created by Bjorn on 2019-09-17
 */
public abstract class BaseChartWithData extends AbstractChartWithData {

    /**
     * 获取所有的超链
     *
     * @return 超链集合
     */
    @Override
    public NameJavaScriptGroup getHotHyperlink(String linkKey) {
        return super.getHotHyperlink(linkKey);
    }

    /**
     * 处理公式，调用formulaProcessor.dealWith方法，覆写该方法需要调用父类的方法。
     */
    @Override
    public void dealFormula(FormulaProcessor formulaProcessor) {
        super.dealFormula(formulaProcessor);
    }

    /**
     * 生成设计器中显示的图片，默认返回一个饼图的图片
     *
     * @return 图片
     */
    @Override
    protected Image designImage(int width, int height, int resolution, ChartWebParaProvider chartWebPara) {
        return super.designImage(width, height, resolution, chartWebPara);
    }

    /**
     * 导出的图片，默认返回空白图片
     *
     * @return 图片
     */
    @Override
    protected Image exportImage(int width, int height, int resolution, ChartWebParaProvider chartWebPara) {
        return super.exportImage(width, height, resolution, chartWebPara);
    }

    /**
     * 生成图表预览的options,子类先调用父类该方法生成JSON后添加自己的属性
     *
     * @return JSON对象
     */
    @Override
    public JSONObject createAttributeConfig(ChartWebParaProvider chartWebPara) {
        return super.createAttributeConfig(chartWebPara);
    }

    /**
     * 返回新增特殊的超链中参数下拉可选项
     *
     * @return 超链参数对象集合
     */
    @Override
    protected HyperLinkPara[] hyperLinkParas() {
        return super.hyperLinkParas();
    }

    /**
     * 读取xml中的基本数据类型属性，子类覆写该方法需要先调用父类的该方法
     */
    @Override
    protected void readAttr(XMLableReader xmLableReader) {
        super.readAttr(xmLableReader);
    }

    /**
     * 将基本数据类型属性写入xml，子类覆写该方法需要先调用父类的该方法
     */
    @Override
    protected void writeAttr(XMLPrintWriter xmlPrintWriter) {
        super.writeAttr(xmlPrintWriter);
    }

    /**
     * 定义图表的ID，与plugin.xml中的chartID对应
     *
     * @return ID
     */
    @Override
    public abstract String getID();

    /**
     * 获取字段集合对象
     *
     * @return 字段集合对象
     */
    public <T extends BaseColumnFieldCollection> T getColumnFieldCollection(Class<T> var1) {
        return super.getFieldCollection(var1);
    }
}
