package com.fanruan.api.util;

import com.fr.stable.AssistUtils;

public class AssistKit {
    /**
     * 比较两个双精度浮点型数据是否相等
     * @param parm1 第一个double数据
     * @param parm2 第二个double数据
     * @return 相等返回true 错误返回false
     */
    public static boolean equals(double parm1, double parm2) {
        return AssistUtils.equals(parm1, parm2);
    }

    /**
     * 比较两个单精度浮点型数据是否相等
     * @param parm1 第一个float数据
     * @param parm2 第二个float数据
     * @return 相等返回true 错误返回false
     */
    public static boolean equals(float parm1, float parm2) {
        return AssistUtils.equals(parm1, parm2);
    }

    /**
     * 判断 float 类型是否相等。
     * <p>
     * 浮点数采用“尾数+阶码”的编码方式，类似于科学计数法的“有效数字+指数”的表示方式。
     * 二进制无法精确表示大部分的十进制小数。
     * <p>
     * 这里首先比较该数二进制是否相等，若不相等则采用近视精度比较，精度可自定义。
     *
     * @param f1    单精度浮点
     * @param f2    单精度浮点
     * @param delta 判断精度
     * @return 是否相等
     * @see #equals(float, float)
     */
    public static boolean equalsFloat(float f1, float f2, float delta) {
        return AssistUtils.equalsFloat(f1, f2 ,delta);
    }

    /**
     * 判断两个Object类型数据是否相等
     * @param parm1 第一个Object数据
     * @param parm2 第二个
     * @return 相等返回true 错误返回false
     */
    public static boolean equals(Object parm1, Object parm2) {
        return AssistUtils.equals(parm1, parm2);
    }

    /**
     * 判断两个int类型数据是否相等
     * @param parm1 第一个int数据
     * @param parm2 第二个int数据
     * @return 相等返回0，如果parm1大于parm2 返回1否则返回-1
     */
    public static int compare(int parm1, int parm2) {
        return AssistUtils.compare(parm1, parm2);
    }

    /**
     * 判断两个long类型数据是否相等
     * @param parm1 第一个long数据
     * @param parm2 第二个long数据
     * @return 相等返回0，如果parm1大于parm2 返回1否则返回-1
     */
    public static int compare(long parm1, long parm2) {
        return AssistUtils.compare(parm1, parm2);
    }

    /**
     * 生成对象的哈希码值
     * @param parm 需要获取hashcode的Object数据
     * @return 返回对象的哈希码值
     */
    public static int hashCode(Object... parm) {
        return AssistUtils.hashCode(parm);
    }

    /**
     * 返回反映这个对象的字符串
     * @param parm 需要获取描述字符串的Object数据
     * @return 返回反映这个对象的字符串
     */
    public static String toString(Object parm) {
        return AssistUtils.toString(parm);
    }

}
